# $Id: fixgroff.awk,v 1.3 2008/08/04 11:13:21 dmw Exp $

# This is be a script to change the groff font files in some ways.

# 1. Change internal name
#	Reason: The inspection of a font file with a name like Nimbus* for
#	Times-Roman may be confusing. E.g. the URW fonts donated to
#	GhostScript have these funny names.
# 2. Remove kerning (for fonts to be fixed pitch).
#	The URW version of Courier claims not to be fixed pitch and has kerns.
# 3. Change name entry
#	Reason: afmtodit insists that the troff name and the file name are
#	the same. This may not be desirable for outputting to a temp file
#	for further processing and installation.

BEGIN	\
	{
		# Variables set on command line:
		# TroffName
		# IntName
		# RemoveKerns
		# FixLigs

		HEADER_ST = 0
		CHARS_ST = 1
		KERN_ST = 2
		State = HEADER_ST

		Headings[HEADER_ST] = ""
		Headings[CHARS_ST] = "charset\n"
		Headings[KERN_ST] = "kernpairs\n"

		EnglishLigatures["ligatures"]	= 0
		EnglishLigatures["fi"]		= 0
		EnglishLigatures["fl"]		= 0
		EnglishLigatures["ff"]		= 0
		EnglishLigatures["ffl"]		= 0
		EnglishLigatures["ffi"]		= 0
		EnglishLigatures["0"]		= 0
	}

/^internalname /	\
	{
		if( IntName != "0" ) $0 = "internalname " IntName
		Store()
		next
	}

/^name /	\
	{
		if( TroffName != "0" ) $0 = "name " TroffName
		Store()
		next
	}

/^ligatures /	\
	{
		$0 = DoLigs($0)
		Store()
		next
	}

/^charset$/	\
	{
		State = CHARS_ST
		Store()
		next
	}

/^kernpairs$/	\
	{
		State = KERN_ST
		Store()
		next
	}

State == CHARS_ST	\
	{
		Store()
		next
	}

	{
		Store()
	}

function Store(){
	# if in kern space and kerning off then skip else print fi
	if( State == KERN_ST && RemoveKerns != "0" ) return
	print $0
}

function DoLigs(s	,n, x, i, ns){
	# Locals: n, x, i, ns

	if( ! FixLigs ) return s
	n = split(s, x, /[ \t]+/)
	ns = ""
	for( i=1; i<=n; i++ ) if( x[i] in EnglishLigatures ) ns = ns " " x[i]
	sub(/^ /, "", ns)
	return ns
}
