# $Id: root.src,v 1.1 2008/07/16 13:31:46 dmw Exp $

# Root of mkgroff.awk

# $Id: general.awk,v 1.3 2008/08/04 11:30:12 dmw Exp $

#  _____________________________________________________
# |							|
# |  Generate calls to afmtodit with suitable options	|
# |  -------- ----- -- -------- ---- -------- -------	|
# |_____________________________________________________|

#=	Index
#=  18	Generate calls to afmtodit with suitable options
#=  26	General Settings
#=  54	Ignore # to end of line
#=  69	Ignore blank lines
#=  86	Option Processing
#= 129	Directories to search for AFM Files
#= 156	Family Lines
#= 223	Style Lines
#= 289	Generate `afmtodit' command
#= 485	End Family Line
#= 506	Utility Functions

# .---------------------.
# |			|
# |  General Settings	|
# |  ------- --------	|
# |			|
# `---------------------'

BEGIN	\
	{
		# Set on command line:
		# GroffDevPS
		# GroffVersion
		# Verbose
		# KeepName
		# TempUseFile

		# Fields are separated by one or more TABs
		FS = "\t+"

		# Boolean type
		false = 0; true = !false

		# Gentle syntax checking
		INFAM_ST	= 1
		OUTFAM_ST	= 2
		InFamily	= OUTFAM_ST
	}

# .-----------------------------.
# |				|
# |   Ignore # to end of line	|
# |   ------ - -- --- -- ----	|
# |				|
# `-----------------------------'

# Also remove white space before the #

/#/	\
	{
		sub(/[ \t]*#.*/, "", $0)
		# drop through
	}

# .---------------------.
# |			|
# | Ignore blank lines	|
# | ------ ----- -----	|
# |			|
# `---------------------'

# Note: DOS files have CR in front of LF, so have
# to take this into account.

/^[ \t\r]*$/	\
	{
		next
	}

# $Id: options.awk,v 1.1 2008/07/16 13:31:46 dmw Exp $

# .---------------------.
# |			|
# |  Option Processing	|
# |  ------ ----------	|
# |			|
# `---------------------'

BEGIN	\
	{
		# Option types
		BOOL_TYPE	= 1
		VAL_TYPE	= 2
		VALOPT_TYPE	= 3

		DEFAULT = " " # Cannot arise via the data
	}

function DoOptions(keys, flags, vals, opts	,x, n, i, f, m, type, key){
	# Locals: x, n, i, f, m, key

	# for( key in keys ) { print "setting opt", key; vals[key] = "" } 
	for( key in keys ) vals[key] = ""
	gsub(/ /, "", opts)
	if( opts == "-" ) return
	n = split(opts, x, /,/)
	for( i=1; i<=n; i++ ){
		m = split(x[i], f, /=/)
		if( m < 1 || m > 2 ) Error("malformed option %s", x[i])
		key = f[1]
		if( ! (key in keys) ) Error("unknown key %s", key)
		type = keys[key]
		if( type == VALOPT_TYPE ){
			vals[key] = m == 1 ? DEFAULT : f[2]
		} else if( m == type ){
			vals[key] = type == VAL_TYPE ? f[2] : flags[key]
		} else {
			Error("wrong param spec for key %s", x[1])
		}
	}
}

# $Id: dirs.awk,v 1.1 2008/07/16 13:31:46 dmw Exp $

# .-------------------------------------.
# |					|
# | Directories to search for AFM Files	|
# | ----------- -- ------ --- --- -----	|
# |					|
# `-------------------------------------'

BEGIN	\
	{
		# Default search directory for afm-files
		Dir = "."
	}

/^Dir\t/	\
	{
		Check(NF, 2, 2, OUTFAM_ST)
		SetDir($2)
		next
	}

function SetDir(dir){
	Dir = dir
	printf "CheckDir %s\n", Dir
}

# $Id: family.awk,v 1.1 2008/07/16 13:31:46 dmw Exp $

# .---------------------.
# |			|
# |    Family Lines	|
# |    ------ -----	|
# |			|
# `---------------------'

BEGIN	\
	{
		# Family options
		FamOptKeys["map"] = VAL_TYPE
		FamOptKeys["enc"] = VAL_TYPE

		FamOptFlag["map"] = "textmap"
		FamOptFlag["enc"] = "text.enc"

		split("", FamOpts)
	}

/^Family\t/	\
	{
		Check(NF, 4, 5, OUTFAM_ST)
		SetFamily($2, $3, $4, NF==5 ? $5 : "")
		next
	}

# The arguments are
# 1.  groffname  The groff family name, eg T for Times;
# 2.  kind       The kind of font; the acceptable values are:
#                  text   A text font
#                  symb   A symbol font (will be special)
# 3.  psname     The PostScript family name. Used only for annotating the
#                generated script.
# 4.  options    Zero or more comma-separated values:
#                  enc=file   The name of an encoding file;
#                             there is a default if kind = text.
#                  map=file    Map file. There is a default if kind = text

function SetFamily(groffname, kind, psname, options){
	printf "# Processing family %s (%s)\n", psname, groffname
	InFamily = INFAM_ST
	DoFamOpts(options)
	DoFamDefaults(kind)
	GroffFam = groffname
	Kind = kind
}

function DoFamOpts(opts){
	DoOptions(FamOptKeys, FamOptFlag, FamOpts, opts)
}

# There are defaults for enc and map for a text font,
# but not for any others.
function DoFamDefaults(kind	,k){
	# Locals: k

	if( kind == "text" ){
		for( k in FamOptKeys ){
			if( FamOpts[k] == "" ) FamOpts[k] = FamOptFlag[k]
		}
	} else if( kind != "symb" ){
		Error("unknown type (%s) for family", kind)
	}
}

# $Id: style.awk,v 1.1 2008/07/16 13:31:46 dmw Exp $

# .-------------.
# |		|
# | Style Lines	|
# | ----- -----	|
# |		|
# `-------------'

BEGIN	\
	{
		# Style options
		# -------------

		# map and enc can over-ride family setting

		StyleOptKeys["ital"]		= VALOPT_TYPE
		StyleOptKeys["angle"]		= VAL_TYPE
		StyleOptKeys["nolig"]		= BOOL_TYPE
		StyleOptKeys["nokern"]		= BOOL_TYPE
		StyleOptKeys["enc"]		= VAL_TYPE
		StyleOptKeys["map"]		= VAL_TYPE
		StyleOptKeys["noadobe"]		= BOOL_TYPE

		StyleOptFlags["ital"]		= "50"
		StyleOptFlags["angle"]		= ""
		StyleOptFlags["nolig"]		= "-n"
		StyleOptFlags["nokern"]		= "-k"	# Unofficial option
		StyleOptFlags["enc"]		= ""
		StyleOptFlags["map"]		= ""
		StyleOptFlags["noadobe"]	= "-x"

		split("", StyleOpts)
	}

/^(-|[A-Za-z]+)\t/	\
	{
		Check(NF, 3, 4, INFAM_ST)
		Style($1, $2, $3, NF==4 ? $4 : "")
		next
	}

function Style(style, afm, opts, psname	,groffname){
	# Locals: groffname

	if( style =="-" ) style = ""
	groffname = GroffFam style
	DoStyleOpts(opts)
	DoStyleDefaults()
	OutCommand(groffname, afm, psname)
}

function DoStyleOpts(opts){
	DoOptions(StyleOptKeys, StyleOptFlags, StyleOpts, opts)
}

# These are available to set some boolean values,
# or to override the Family options.
function DoStyleDefaults(	k){
	# Locals: k

	for( k in StyleOptKeys ){
		if( StyleOpts[k] == DEFAULT ) StyleOpts[k] = StyleOptFlags[k]
	}
}

# $Id: command.awk,v 1.5 2008/08/04 11:29:11 dmw Exp $

# .-----------------------------.
# |				|
# | Generate `afmtodit' command	|
# | -------- ---------- -------	|
# |				|
# `-----------------------------'

# y	afmtodit
#	[ options ]
# y		-a angle	Set slant parameter
# y		-c		Always use (if V19)
# -		-d desc		Does anyone ever use a different DESC?
#				V18 does not look in system directory
# y		-e encoding	Default is encoding implied by afm-file
#				If given, search in . then devps
# y		-i number	Generate italic correction
# y		-m		No negative italic corrections (if V19)
# y		-n		No ligatures
# y		-s		Special font
# y		-x		No builtin Adobe glyph names (if V19)
#        (next two are my additions to afmtodit, but I'll not use them)
# y		-k		Omit kerns (if V19)
# y		-f name		Change (lie) about name (if V19)
#
# y	afm-file
# y	map-file
# y	groff-name

BEGIN	\
	{
		# GroffVersion is defined on command line.
		CLEAR	= 1
		APPEND	= 2
		FINISH	= 3

		Length = 0
		OldGroff = GroffVersion <= "1.18.999"
		# OldGroff = true ### XXX TEMP for testing
	}

# Since V19 afmtodit looks for its files in . or .../devps. Therefore
# in case a system still uses an earlier version (eg debian-based) generate
# search commands for these files.
function OutCommand(groffname, afm, psname, x, y, z){
	# Locals: x, y, z
	
	if( Verbose ){
 		z = sprintf("echo \"%s: %s (%s)\"", groffname, psname, afm)
		Command(z, CLEAR)
		Command("", FINISH)
	}

	Command(sprintf("afmtodit%s", OldGroff ? "" : " -c"), CLEAR)

	# DESC file
	# ---------

	if ( OldGroff ){
		z = "-d " GenSearch("DESC", "desc_file")
		Command(z, APPEND)
	}

	# Encoding (-e flag)
	# -------- ---------

	x = StyleOpts["enc"]
	y = FamOpts["enc"]
	z = ""
	if( x != "" ) z = x
	else if( y != "" ) z = y
	if( z != "" ){
		z = "-e " GenSearch(z, "enc_file")
		Command(z, APPEND)
	}

	# Italic correction
	# ------ ----------

	x = StyleOpts["ital"]
	if( x != "" ) z = sprintf("-i %d", x)
	else z = sprintf("%s-i0", OldGroff ? "" : "-m ")
	Command(z, APPEND)

	# Slant (-a angle)
	# ----- ----------

	x = StyleOpts["angle"]
	if( x != "" ) Command(sprintf("-a %s", x), APPEND)

	# Special, ligatures, kerns and glyph list
	# -------  ---------  ----- --- ----- ----

	if( Kind == "symb" ) BoolCommand("-s")
	# If `afmtodit' has the -k option use this, else postprocess
	# if necessary.
	# BoolCommand(StyleOpts["nokern"])
	BoolCommand(StyleOpts["nolig"])
	if( ! OldGroff ) BoolCommand(StyleOpts["noadobe"])

	# False name
	# ----- ----

	# If `afmtodit' has the -f option use this, else postprocess
	# if necessary. May be a good idea anyway, as `afmtodit' has
	# the wekness that the troff font name and the file name it
	# creates are identical.
	# if( psname != "" && ! KeepName ) \
	#	Command(sprintf("-f %s", psname), APPEND)

	# Afm file
	# --- ----

	z = afm
	if( ! match(afm, /[.][aA][fF][mM]$/) ) z = z ".afm"

	Command(Dir "/" z, APPEND)

	# Map file
	# --- ----

	x = StyleOpts["map"]
	y = FamOpts["map"]
	z = ""
	if( x != "" ) z = x
	else if( y != "" ) z = y
	if( z != "" ){
		z = GenSearch(z, "map_file")
		Command(z, APPEND)
	}
	else Error("missing map file%s", "")

	# Groff font name
	# ----- ---- ----

	Command(TempUseFile, APPEND)

	Command("", FINISH)

	# Command to fix the result
	# ------- -- --- --- ------

	Command("Fix", CLEAR)
	Command(groffname, APPEND)
	z = sprintf("%s",  psname != "" && ! KeepName ? psname : "0" )
	Command(z, APPEND)
	z = StyleOpts["nokern"]
	if( z == "" ) z = "0"
	Command(z, APPEND)
	Command( OldGroff ? "1" : "0", FINISH)
}

function GenSearch(file, name){
	if( OldGroff && ! match(name, /^[/]/) ){
		printf "%s=`Search %s`\n", name, file
		return sprintf("${%s}", name)
	} else {
		return file
	}
}

function BoolCommand(s){
	if( s != "" ) Command(s, APPEND)
}

function Command(com, new	,len, i){
	# Locals: len

	if( new == CLEAR ){
		if( Length != 0 ) Error("misuse of CLEAR (prog err)%s", "")
		Line = ""
		Length = 0
		NumLines = 0
		new = APPEND
	}
	len = length(com)
	if( Length == 0 ){
		Line = com
		Length = len
	} else if( Length + len + 3 > 79 ){
		Lines[++NumLines] = Line
		Line = com
		Length = len
	} else {
		Line = sprintf("%s %s", Line, com)
		Length += len + 1
	}
	if( new == FINISH ){
		if( Length > 0 ) Lines[++NumLines] = Line
		for ( i=1; i<=NumLines; i++ ) printf "%s%s\n", Lines[i],
			i == NumLines ? "" : " \\"
		Length = 0
	}
}

# $Id: endfam.awk,v 1.1 2008/07/16 13:31:46 dmw Exp $

# .---------------------.
# |			|
# |   End Family Line	|
# |   --- ------ ----	|
# |			|
# `---------------------'

/^(---)($|\t+)/	\
	{
		Check(NF, 1, 10, true)
		InFamily = OUTFAM_ST
		next
	}

	{
		# Other lines --- error
		Error("unknown line%s", "")
	}

# $Id: utils.awk,v 1.2 2008/08/03 12:01:37 dmw Exp $

# .---------------------.
# |			|
# |  Utility Functions	|
# |  ------- ---------	|
# |			|
# `---------------------'

function Check(nfields, n1, n2, state){
	if( (nfields<n1) || (nfields>n2) ){
		Error("wrong number of fields (%d)", nfields)
	}
	if( InFamily != state ) Error("Missing `Family' or `---'%s", "")
}

# Exactly one format and value
function Error(s, val	,errfile){
	# Locals: errfile

	errfile = "/dev/stderr"
	printf "Error: line %d: [%s]\n", FNR, $0 > errfile
	printf s, val >  errfile
	printf "\n" >  errfile
	close(errfile)
	exit 1
}
