#!/bin/sh

# $Id: mkgroff.sh,v 1.6 2008/08/04 11:13:21 dmw Exp $

#  _____________________________________________
# |						|
# |    CREATE GROFF FONTS FOR USE WITH DEVPS	|
# |    ------ ----- ----- --- --- ---- -----	|
# |_____________________________________________|

# Make a script to create groff fonts, using a given database.
# Uses `afmtodit' and the .afm files.

thisprog="$0"
set -e -u

VERSION=1
REVISION=6
DATE=2/08/2008

# Temporary files
#	TEMP_SCRIPT	Generated shell script to call other things
#	TEMPUSE		Output from `afmtodit'
#	TEMPFONT	Massaged version of TEMPUSE

TEMP_SCRIPT=mkgroff-tmp.$$
    TEMPUSE=x-tempuse
   TEMPFONT=x-tempfont

AWK=@AWK@
LIBEXEC_DIR=@LIBEXEC_DIR@

AWKSCRIPT=mkgroff.awk
# AWKPATH=.:/usr/local/libexec # Not available for Posix-compliant awk
# export AWKPATH
if [ ! -f ${AWKSCRIPT} ] ; then
	AWKSCRIPT=${LIBEXEC_DIR}/${AWKSCRIPT}
fi
FIXSCRIPT=fixgroff.awk
if [ ! -f ${FIXSCRIPT} ] ; then
	FIXSCRIPT=${LIBEXEC_DIR}/${FIXSCRIPT}
fi

#  _____________________
# |			|
# |   Shell Functions	|
# |   ----- ---------	|
# |_____________________|

Error(){
	echo "$thisprog: $*" 1>&2
	exit 1
}

Help(){
cat <<'ENDHELP'
Usage: $thisprog [ options ] infile

The options are as follows (a single - may be used in place of --):

    -h|--help		Print this help message and exit
       --version	Print version information and exit
    -b|--build		Build the generated fonts in given directory
			(default BUILD)
    -k|--keep		Keep the `FontName' given in the AFM file
    -n|--norun		Do not run the generated script, print on stdout
       --verb		Echo the font and files before processing each one
ENDHELP
}

SetInput(){
	if [ ! -z "$infile" ] ; then
		Error "may not specify input file $infile twice"
	fi
	infile="$1"
}

Version(){
	echo "$thisprog version ${VERSION}.${REVISION} (${DATE})"
}

# .-----------------------------.
# |				|
# | Invoked by generated script	|
# | ------- -- --------- ------	|
# |				|
# `-----------------------------'

CheckDir(){
	if [ ! -d "$1" ] ; then
		Error "$1" is not a directory
	fi
}

# Search path is . devps devps/generate
Search(){
	for file in $1 $DEVPS_DIR/$1 $DEVPS_DIR/generate/$1 ; do
		if [ -f $file ] ; then
			echo $file
			return 0
		fi
	done
	Error "file $1 not found"
}

MakeScript(){
	${AWK} \
	-v GroffDevPS=${DEVPS_DIR} \
	-v GroffVersion=${GROFF_VERSION} \
	-v Verbose=${verb} \
	-v KeepName=${keep} \
	-v TempUseFile=${TEMPUSE} \
	-f ${AWKSCRIPT} \
	${infile}
}

# If afmtodit is modified with -f and -k options, can remove
# `fixgroff', but keep it for older versions of troff.

Fix(){
	${AWK} \
	-v   TroffName=$1	\
	-v     IntName=$2	\
	-v RemoveKerns=$3	\
	-v     FixLigs=$4	\
	-f ${FIXSCRIPT}		\
	${TEMPUSE}		\
	> ${TEMPFONT}

	mv ${TEMPFONT} ${builddir}/$1
}

# =======================================================================
# PROGRAM PROPER BEGINS HERE

#  _____________________
# |			|
# |  Process Arguments	|
# |  ------- ---------	|
# |_____________________|

thisprog=`basename $0`

infile=""
builddir="BUILD"
keep=0
norun=0
verb=0

ARGS=`getopt -a -o hb:kn -l help,version,build:,keep,norun,verb -- "$@"`
if [ $? -ne 0 ] ; then
	exit 1
fi
eval set -- "$ARGS"
while [ $# -ne 0 ] ; do
	case $1 in
		-h|--help)	Help; exit 0 ;;
		   --version)	Version ; exit 0 ;;
		-b|--build)	builddir="$2"; shift ;;
		-k|--keep)	keep=1 ;;
		-n|--norun)	norun=1 ;;
		   --verb)	verb=1 ;;
		--)		;;
		-*)		echo "unknown option $1" 1>&2; exit 1 ;;
		*)		SetInput $1 ;;
	esac
	shift
done

# Input file must be specified and must exist
if [ -z "$infile" ] ; then
	Error "must specify an input file"
fi

if [ ! -f "$infile" ] ; then
	Error "input file $infile does not exist"
fi

# Find the installed devps directory, where the files
#
#	DESC, text.enc, textmap
#
# are kept:

GROFF_VERSION=`groff -v | sed -n -e 's/^GNU groff version \(.*\)$/\1/p'`

# May have installed groff in /usr or /usr/local, give priority to latter.

if [ -d /usr/local/share/groff/${GROFF_VERSION} ] ; then
	local=/local
else
	local=""
fi
DEVPS_DIR=/usr${local}/share/groff/${GROFF_VERSION}/font/devps

if [ $norun -eq 1 ] ; then
	MakeScript
	exit 0
else
	MakeScript > ${TEMP_SCRIPT}
fi

mkdir -p $builddir
. ${TEMP_SCRIPT}

rm -f ${TEMP_SCRIPT}
rm -f ${TEMPUSE}
rm -f ${TEMPFONT}
